@@ -24,12 +24,15 @@
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <string.h>
 #include <stdlib.h>
 #include <gtk/gtksignal.h>
 #include <gtk/gtkhbox.h>
 #include <gtk/gtkvbox.h>
-#include <gtk/gtkclist.h>
 #include <gtk/gtkentry.h>
 #include <gtk/gtkframe.h>
 #include <gtk/gtkscrolledwindow.h>
@@ -37,6 +40,11 @@
 #include <gtk/gtkimage.h>
 #include <gtk/gtkdialog.h>
 #include <gtk/gtkstock.h>
+#include <gtk/gtklabel.h>
+#include <gtk/gtktreeview.h>
+#include <gtk/gtkliststore.h>
+#include <gtk/gtktreeselection.h>
+#include <gtk/gtkcellrenderertext.h>
 #include <libgnomeprint/gnome-pgl.h>
 #include "gnome-print-i18n.h"
 #include "gnome-font-dialog.h"
@@ -90,11 +98,15 @@ static void gnome_font_selection_class_init (GnomeFontSelectionClass *klass);
 static void gnome_font_selection_init (GnomeFontSelection *fontsel);
 static void gnome_font_selection_destroy (GtkObject *object);
 
-static void gnome_font_selection_select_family (GtkCList * clist, gint row, gint column, GdkEvent * event, gpointer data);
-static void gnome_font_selection_select_style (GtkCList * clist, gint row, gint column, GdkEvent * event, gpointer data);
+static void gnome_font_selection_select_family (GtkTreeSelection *selection, gpointer data);
+static void gnome_font_selection_select_style (GtkTreeSelection *selection, gpointer data);
 static void gnome_font_selection_select_size (GtkEditable * editable, gpointer data);
 static void gnome_font_selection_fill_families (GnomeFontSelection * fontsel);
 static void gnome_font_selection_fill_styles (GnomeFontSelection * fontsel);
+static gboolean find_row_to_select_cb (GtkTreeModel *model,
+				       GtkTreePath *path,
+				       GtkTreeIter *iter,
+				       gpointer user_data);
 
 static GtkHBoxClass *gfs_parent_class = NULL;
 static guint gfs_signals[LAST_SIGNAL] = {0};
@@ -145,7 +157,11 @@ static void
 gnome_font_selection_init (GnomeFontSelection * fontsel)
 {
 	static GList * sizelist = NULL;
-	GtkWidget * f, * sw, * cl, * vb, * hb, * c, * l;
+	GtkWidget * f, * sw, * tv, * vb, * hb, * c, * l;
+	GtkListStore *store;
+	GtkTreeSelection *selection;
+	GtkTreeViewColumn *col;
+	GtkCellRenderer *rend;
 
 	gtk_box_set_homogeneous ((GtkBox *) fontsel, TRUE);
 	gtk_box_set_spacing ((GtkBox *) fontsel, 4);
@@ -159,21 +175,34 @@ gnome_font_selection_init (GnomeFontSelection * fontsel)
 	sw = gtk_scrolled_window_new (NULL, NULL);
 	gtk_container_set_border_width ((GtkContainer *) sw, 4);
 	gtk_scrolled_window_set_policy ((GtkScrolledWindow *) sw, GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
+	gtk_scrolled_window_set_shadow_type ((GtkScrolledWindow *) sw,
+					     GTK_SHADOW_IN);
 	gtk_widget_show (sw);
 	gtk_container_add ((GtkContainer *) f, sw);
 
-	cl = gtk_clist_new (1);
-	gtk_clist_set_selection_mode ((GtkCList *) cl, GTK_SELECTION_SINGLE);
-	gtk_clist_column_titles_hide ((GtkCList *) cl);
+	store = gtk_list_store_new (1, G_TYPE_STRING);
+	tv = gtk_tree_view_new_with_model ((GtkTreeModel*) store);
+	selection = gtk_tree_view_get_selection ((GtkTreeView*) tv);
+	gtk_tree_selection_set_mode ((GtkTreeSelection*) selection,
+				     GTK_SELECTION_SINGLE);
+	g_object_unref ((GObject*) store);
+	gtk_tree_view_set_headers_visible ((GtkTreeView*) tv, FALSE);
+
+	rend = gtk_cell_renderer_text_new ();
+	col = gtk_tree_view_column_new_with_attributes (NULL, rend,
+							"text", 0,
+							NULL);
+	gtk_tree_view_column_set_sizing (col, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
+	gtk_tree_view_append_column (GTK_TREE_VIEW (tv), col);
 #if 0
-	gtk_widget_set_usize (cl, 64, 128);
+	gtk_widget_set_usize (tv, 64, 128);
 #endif
-	gtk_clist_set_column_auto_resize ((GtkCList *) cl, 0, TRUE);
-	gtk_widget_show (cl);
-	gtk_signal_connect (GTK_OBJECT (cl), "select_row",
-			    GTK_SIGNAL_FUNC (gnome_font_selection_select_family), fontsel);
-	gtk_container_add ((GtkContainer *) sw, cl);
-	fontsel->family = cl;
+	gtk_widget_show (tv);
+	g_signal_connect ((GObject*) selection, "changed",
+			  GTK_SIGNAL_FUNC (gnome_font_selection_select_family),
+			  fontsel);
+	gtk_container_add ((GtkContainer *) sw, tv);
+	fontsel->family = tv;
 	fontsel->selectedfamily = NULL;
 
 	/* Fontbox */
@@ -199,21 +228,34 @@ gnome_font_selection_init (GnomeFontSelection * fontsel)
 
 	sw = gtk_scrolled_window_new (NULL, NULL);
 	gtk_scrolled_window_set_policy ((GtkScrolledWindow *) sw, GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
+	gtk_scrolled_window_set_shadow_type ((GtkScrolledWindow *) sw,
+					     GTK_SHADOW_IN);
 	gtk_widget_show (sw);
 	gtk_box_pack_start ((GtkBox *) vb, sw, TRUE, TRUE, 0);
 
-	cl = gtk_clist_new (1);
-	gtk_clist_set_selection_mode ((GtkCList *) cl, GTK_SELECTION_SINGLE);
-	gtk_clist_column_titles_hide ((GtkCList *) cl);
+	store = gtk_list_store_new (1, G_TYPE_STRING);
+	tv = gtk_tree_view_new_with_model ((GtkTreeModel*) store);
+	selection = gtk_tree_view_get_selection ((GtkTreeView*) tv);
+	gtk_tree_selection_set_mode ((GtkTreeSelection*) selection,
+				     GTK_SELECTION_SINGLE);
+	g_object_unref ((GObject*) store);
+	gtk_tree_view_set_headers_visible ((GtkTreeView*) tv, FALSE);
+
+	rend = gtk_cell_renderer_text_new ();
+	col = gtk_tree_view_column_new_with_attributes (NULL, rend,
+							"text", 0,
+							NULL);
+	gtk_tree_view_column_set_sizing (col, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
+	gtk_tree_view_append_column (GTK_TREE_VIEW (tv), col);
 #if 0
-	gtk_widget_set_usize (cl, 64, 64);
+	gtk_widget_set_usize (tv, 64, 64);
 #endif
-	gtk_clist_set_column_auto_resize ((GtkCList *) cl, 0, TRUE);
-	gtk_widget_show (cl);
-	gtk_signal_connect (GTK_OBJECT (cl), "select_row",
-			    GTK_SIGNAL_FUNC (gnome_font_selection_select_style), fontsel);
-	gtk_container_add ((GtkContainer *) sw, cl);
-	fontsel->style = cl;
+	gtk_widget_show (tv);
+	g_signal_connect ((GObject*) selection, "changed",
+			  GTK_SIGNAL_FUNC (gnome_font_selection_select_style),
+			  fontsel);
+	gtk_container_add ((GtkContainer *) sw, tv);
+	fontsel->style = tv;
 	fontsel->selectedface = NULL;
 
 	/* Sizebox */
@@ -285,27 +327,45 @@ GtkWidget *
 gnome_font_selection_new ()
 {
 	GnomeFontSelection * fontsel;
-  
+	GtkTreeModel *model;
+	GtkTreeIter iter;
+
 	fontsel = gtk_type_new (GNOME_TYPE_FONT_SELECTION);
   
 	gnome_font_selection_fill_families (fontsel);
 
 	/* Select first font in family list */
 
-	gtk_clist_select_row ((GtkCList *) fontsel->family, 0, 0);
+	model = gtk_tree_view_get_model ((GtkTreeView*) fontsel->family);
+	if (gtk_tree_model_get_iter_first (model, &iter))
+	  {
+	    GtkTreeSelection *selection;
+
+	    selection = gtk_tree_view_get_selection ((GtkTreeView*) fontsel->family);
+	    gtk_tree_selection_select_iter (selection, &iter);
+	  }
 
 	return GTK_WIDGET (fontsel);
 }
 
 static void
-gnome_font_selection_select_family (GtkCList * clist, gint row, gint column, GdkEvent * event, gpointer data)
+gnome_font_selection_select_family (GtkTreeSelection *selection, gpointer data)
 {
 	GnomeFontSelection * fontsel;
-	gchar * familyname;
+	GtkTreeView *tree_view;
+	GtkTreeModel *model;
+	GtkTreeIter iter;
+	GValue value = { 0 };
+	const gchar * familyname;
 
 	fontsel = GNOME_FONT_SELECTION (data);
 
-	gtk_clist_get_text (clist, row, column, &familyname);
+	tree_view = gtk_tree_selection_get_tree_view (selection);
+	if (!gtk_tree_selection_get_selected (selection, &model, &iter))
+		return;
+
+	gtk_tree_model_get_value (model, &iter, 0, &value);
+	familyname = g_value_get_string (&value);
 
 	if (fontsel->selectedfamily) {
 		g_free (fontsel->selectedfamily);
@@ -317,20 +377,31 @@ gnome_font_selection_select_family (GtkCList * clist, gint row, gint column, Gdk
 		fontsel->selectedfamily = NULL;
 	}
 
+	g_value_unset(&value);
+
 	gnome_font_selection_fill_styles (fontsel);
 }
 
 static void
-gnome_font_selection_select_style (GtkCList * clist, gint row, gint column, GdkEvent * event, gpointer data)
+gnome_font_selection_select_style (GtkTreeSelection *selection, gpointer data)
 {
 	GnomeFontSelection * fontsel;
-	gchar * style;
+	GtkTreeView *tree_view;
+	GtkTreeModel *model;
+	GtkTreeIter iter;
+	GValue value = { 0 };
+	const gchar * style;
 
 	fontsel = GNOME_FONT_SELECTION (data);
 
 	if (!fontsel->selectedfamily) return;
 
-	gtk_clist_get_text (clist, row, column, &style);
+	tree_view = gtk_tree_selection_get_tree_view (selection);
+	if (!gtk_tree_selection_get_selected (selection, &model, &iter))
+		return;
+
+	gtk_tree_model_get_value (model, &iter, 0, &value);
+	style = g_value_get_string (&value);
 
 	if (fontsel->selectedface) gnome_font_face_unref (fontsel->selectedface);
 	fontsel->selectedface = gnome_font_face_find_from_family_and_style (fontsel->selectedfamily, style);
@@ -338,6 +409,8 @@ gnome_font_selection_select_style (GtkCList * clist, gint row, gint column, GdkE
 	if (fontsel->selectedfont) gnome_font_unref (fontsel->selectedfont);
 	fontsel->selectedfont = gnome_font_face_get_font_default (fontsel->selectedface, fontsel->selectedsize);
 
+	g_value_unset(&value);
+
 	gtk_signal_emit (GTK_OBJECT (fontsel), gfs_signals[FONT_SET], fontsel->selectedfont);
 }
 
@@ -367,18 +440,21 @@ static void
 gnome_font_selection_fill_families (GnomeFontSelection * fontsel)
 {
 	GList * families, * l;
+	GtkListStore *store;
 
 	families = gnome_font_family_list ();
 	g_return_if_fail (families != NULL);
 
-	gtk_clist_freeze ((GtkCList *) fontsel->family);
-	gtk_clist_clear ((GtkCList *) fontsel->family);
+	store = (GtkListStore*) gtk_tree_view_get_model ((GtkTreeView*) fontsel->family);
+
+	gtk_list_store_clear (store);
 
 	for (l = families; l != NULL; l = l->next) {
-		gtk_clist_append ((GtkCList *) fontsel->family, (gchar **) &l->data);
-	}
+		GtkTreeIter iter;
 
-	gtk_clist_thaw ((GtkCList *) fontsel->family);
+		gtk_list_store_append (store, &iter);
+		gtk_list_store_set (store, &iter, 0, l->data, -1);
+	}
 
 	gnome_font_family_list_free (families);
 }
@@ -387,23 +463,33 @@ static void
 gnome_font_selection_fill_styles (GnomeFontSelection * fontsel)
 {
 	GList * styles, * l;
+	GtkListStore *store;
+	GtkTreeIter iter;
 
-	gtk_clist_freeze ((GtkCList *) fontsel->style);
-	gtk_clist_clear ((GtkCList *) fontsel->style);
+	store = (GtkListStore*) gtk_tree_view_get_model ((GtkTreeView*) fontsel->style);
+
+	gtk_list_store_clear (store);
 
 	if (fontsel->selectedfamily) {
 		styles = gnome_font_style_list (fontsel->selectedfamily);
 		for (l = styles; l != NULL; l = l->next) {
-			gtk_clist_append ((GtkCList *) fontsel->style, (gchar **) &l->data);
+			GtkTreeIter iter;
+
+			gtk_list_store_append (store, &iter);
+			gtk_list_store_set (store, &iter, 0, l->data, -1);
 		}
 		gnome_font_style_list_free (styles);
 	}
 
-	gtk_clist_thaw ((GtkCList *) fontsel->style);
-
 	/* Select first font in style list */
 
-	gtk_clist_select_row ((GtkCList *) fontsel->style, 0, 0);
+	if (gtk_tree_model_get_iter_first ((GtkTreeModel *) store, &iter))
+	  {
+	    GtkTreeSelection *selection;
+
+	    selection = gtk_tree_view_get_selection ((GtkTreeView*) fontsel->style);
+	    gtk_tree_selection_select_iter (selection, &iter);
+	  }
 }
 
 
@@ -439,14 +525,22 @@ gnome_font_selection_get_face (GnomeFontSelection * fontsel)
 	return fontsel->selectedface;
 }
 
+typedef struct _GnomeFontSelectionSetFontData GnomeFontSelectionSetFontData;
+struct _GnomeFontSelectionSetFontData
+{
+	GtkTreeSelection *selection;
+	const gchar *row_to_select;
+};
+
 void
 gnome_font_selection_set_font (GnomeFontSelection * fontsel, GnomeFont * font)
 {
 	const GnomeFontFace * face;
 	const gchar * familyname, * stylename;
 	gdouble size;
+	GtkTreeModel *model;
+	GnomeFontSelectionSetFontData data;
 	gchar b[32];
-	gint rows, row;
 
 	g_return_if_fail (fontsel != NULL);
 	g_return_if_fail (GNOME_IS_FONT_SELECTION (fontsel));
@@ -458,21 +552,17 @@ gnome_font_selection_set_font (GnomeFontSelection * fontsel, GnomeFont * font)
 	stylename = gnome_font_face_get_species_name (face);
 	size = gnome_font_get_size (font);
 
-	rows = ((GtkCList *) fontsel->family)->rows;
-	for (row = 0; row < rows; row++) {
-		gchar * text;
-		gtk_clist_get_text ((GtkCList *) fontsel->family, row, 0, &text);
-		if (strcmp (text, familyname) == 0) break;
-	}
-	gtk_clist_select_row ((GtkCList *) fontsel->family, row, 0);
+	/* Select the matching family. */
+	model = gtk_tree_view_get_model ((GtkTreeView*) fontsel->family);
+	data.selection = gtk_tree_view_get_selection ((GtkTreeView*) fontsel->family);
+	data.row_to_select = familyname;
+	gtk_tree_model_foreach (model, find_row_to_select_cb, &data);
 
-	rows = ((GtkCList *) fontsel->style)->rows;
-	for (row = 0; row < rows; row++) {
-		gchar * text;
-		gtk_clist_get_text ((GtkCList *) fontsel->style, row, 0, &text);
-		if (strcmp (text, stylename) == 0) break;
-	}
-	gtk_clist_select_row ((GtkCList *) fontsel->style, row, 0);
+	/* Select the matching style. */
+	model = gtk_tree_view_get_model ((GtkTreeView*) fontsel->style);
+	data.selection = gtk_tree_view_get_selection ((GtkTreeView*) fontsel->style);
+	data.row_to_select = stylename;
+	gtk_tree_model_foreach (model, find_row_to_select_cb, &data);
 
 	g_snprintf (b, 32, "%2.1f", size);
 	b[31] = '\0';
@@ -480,6 +570,31 @@ gnome_font_selection_set_font (GnomeFontSelection * fontsel, GnomeFont * font)
 	fontsel->selectedsize = size;
 }
 
+static gboolean
+find_row_to_select_cb (GtkTreeModel *model,
+		       GtkTreePath *path,
+		       GtkTreeIter *iter,
+		       gpointer user_data)
+{
+  GnomeFontSelectionSetFontData *data = user_data;
+  gchar *family;
+  gint cmp;
+
+  gtk_tree_model_get (model, iter, 0, &family, -1);
+
+  cmp = strcmp (family, data->row_to_select);
+  g_free (family);
+
+  if (cmp == 0)
+    {
+      gtk_tree_selection_select_path (data->selection, path);
+      return TRUE;
+    }
+
+  return FALSE;
+}
+
+
 #if 0
 void
 gnome_font_selection_bind_editable_enters (GnomeFontSelection * gfs, GnomeDialog * dialog)
@@ -555,7 +670,6 @@ static void
 gnome_font_preview_init (GnomeFontPreview * preview)
 {
 	GtkWidget * w;
-	GtkStyle * style;
 
 	w = (GtkWidget *) preview;
 
