@@ -25,13 +25,19 @@
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include "gnome-print-i18n.h"
 
 #include <time.h>
 #include <gdk/gdkkeysyms.h>
+#include <gtk/gtksignal.h>
 #include <gtk/gtkdialog.h>
 #include <gtk/gtknotebook.h>
 #include <gtk/gtkvbox.h>
+#include <gtk/gtkhbox.h>
 #include <gtk/gtkframe.h>
 #include <gtk/gtktable.h>
 #include <gtk/gtkradiobutton.h>
@@ -39,7 +45,6 @@
 #include <gtk/gtkentry.h>
 #include <gtk/gtkspinbutton.h>
 #include <gtk/gtkstock.h>
-#include <libgnomeui/gnome-dateedit.h>
 
 #include <libgnomeprint/gnome-print-config.h>
 
@@ -66,19 +71,18 @@ struct _GnomePrintDialogClass {
 };
 
 static void gnome_print_dialog_class_init (GnomePrintDialogClass *class);
-static void gnome_print_dialog_init (GnomePrintDialog *gspaper);
+static void gnome_print_dialog_init (GnomePrintDialog *dialog);
 
-static GtkWidget *gpd_create_job_page (void);
+static GtkWidget *gpd_create_job_page (GnomePrintDialog *gpd);
 
-static GtkDialog *parent_class;
+static GtkDialogClass *parent_class;
 
 GtkType
 gnome_print_dialog_get_type (void)
 {
-	static GtkType select_paper_type = 0;
-  
-	if (!select_paper_type) {
-		GtkTypeInfo select_paper_info = {
+	static GtkType type = 0;
+	if (!type) {
+		GtkTypeInfo info = {
 			"GnomePrintDialog",
 			sizeof (GnomePrintDialog),
 			sizeof (GnomePrintDialogClass),
@@ -86,10 +90,9 @@ gnome_print_dialog_get_type (void)
 			(GtkObjectInitFunc) gnome_print_dialog_init,
 			NULL, NULL, NULL
 		};
-		select_paper_type = gtk_type_unique (GTK_TYPE_DIALOG, &select_paper_info);
+		type = gtk_type_unique (GTK_TYPE_DIALOG, &info);
 	}
-  
-	return select_paper_type;
+	return type;
 }
 
 static void
@@ -113,6 +116,7 @@ gnome_print_dialog_class_init (GnomePrintDialogClass *class)
 	GtkObjectClass *object_class;
 	
 	object_class = (GtkObjectClass *) class;
+
 	parent_class = gtk_type_class (GTK_TYPE_DIALOG);
 
 	object_class->destroy = gnome_print_dialog_destroy;
@@ -129,7 +133,7 @@ gnome_print_dialog_init (GnomePrintDialog *gpd)
 		gtk_widget_show (gpd->notebook);
 		gtk_box_pack_start (GTK_BOX (GTK_DIALOG (gpd)->vbox), gpd->notebook, TRUE, TRUE, 0);
 		/* Job page */
-		gpd->job = gpd_create_job_page ();
+		gpd->job = gpd_create_job_page (gpd);
 		gtk_widget_show (gpd->job);
 		l = gtk_label_new (_("Job"));
 		gtk_widget_show (l);
@@ -160,8 +164,17 @@ gnome_print_dialog_init (GnomePrintDialog *gpd)
 	}
 }
 
+static void
+gpd_copies_set (GnomePrintCopiesSelection *gpc, gint copies, gboolean collate, GnomePrintDialog *gpd)
+{
+	if (gpd->config) {
+		gnome_print_config_set_int (gpd->config, GNOME_PRINT_KEY_NUM_COPIES, copies);
+		gnome_print_config_set_boolean (gpd->config, GNOME_PRINT_KEY_COLLATE, collate);
+	}
+}
+
 static GtkWidget *
-gpd_create_job_page (void)
+gpd_create_job_page (GnomePrintDialog *gpd)
 {
 	GtkWidget *hb, *vb, *f, *c;
 
@@ -177,6 +190,7 @@ gpd_create_job_page (void)
 	gtk_object_set_data (GTK_OBJECT (hb), "range", f);
 
 	c = gnome_print_copies_selection_new ();
+	if (gpd) gtk_signal_connect (GTK_OBJECT (c), "copies_set", GTK_SIGNAL_FUNC (gpd_copies_set), gpd);
 	gtk_widget_hide (c);
 	gtk_box_pack_start (GTK_BOX (vb), c, FALSE, FALSE, PAD);
 	gtk_object_set_data (GTK_OBJECT (hb), "copies", c);
@@ -363,7 +377,7 @@ gnome_print_dialog_construct_range_any (GnomePrintDialog *gpd, gint flags, GtkWi
 
 	f = gtk_object_get_data (GTK_OBJECT (gpd->job), "range");
 	g_return_if_fail (f != NULL);
-	r = gtk_object_get_data (GTK_OBJECT (r), "range");
+	r = gtk_object_get_data (GTK_OBJECT (f), "range");
 	if (r) gtk_container_remove (GTK_CONTAINER (f), r);
 
 	r = gpd_create_range (flags, range_widget, currentlabel, rangelabel);
@@ -453,13 +467,13 @@ gnome_print_dialog_get_range (GnomePrintDialog *gpd)
 	r = gtk_object_get_data (GTK_OBJECT (f), "range");
 	g_return_val_if_fail (r != NULL, 0);
 
-	b = gtk_object_get_data (GTK_OBJECT (b), "current");
+	b = gtk_object_get_data (GTK_OBJECT (r), "current");
 	if (b && GTK_IS_TOGGLE_BUTTON (b) && gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (b))) return GNOME_PRINT_RANGE_CURRENT;
-	b = gtk_object_get_data (GTK_OBJECT (b), "all");
+	b = gtk_object_get_data (GTK_OBJECT (r), "all");
 	if (b && GTK_IS_TOGGLE_BUTTON (b) && gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (b))) return GNOME_PRINT_RANGE_ALL;
-	b = gtk_object_get_data (GTK_OBJECT (b), "range");
+	b = gtk_object_get_data (GTK_OBJECT (r), "range");
 	if (b && GTK_IS_TOGGLE_BUTTON (b) && gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (b))) return GNOME_PRINT_RANGE_RANGE;
-	b = gtk_object_get_data (GTK_OBJECT (b), "selection");
+	b = gtk_object_get_data (GTK_OBJECT (r), "selection");
 	if (b && GTK_IS_TOGGLE_BUTTON (b) && gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (b))) return GNOME_PRINT_RANGE_SELECTION;
 
 	return 0;
